#!/bin/sh
# +--------------------------------------------------------------------------+
# | upgrade_script.sh                                                        |
# +--------------------------------------------------------------------------+
# +--------------------------------------------------------------------------+
# | Configuration parameters                                                 |
# +--------------------------------------------------------------------------+
#
# 13.01.2013	Creation d'une version simplifi pour copier des fichiers sans rechargement de 'main'

SCRIPT_VERSION=0.0.1
DEVICE=/dev/sda1
LOGFILE=upgrade_script.log
LISTFILE=upgrade_list.txt
UPGRADE_SCRIPT=upgrade_script.sh
P_500_UPGRADE_PATTERN=p_500_upgrade_
P_800_UPGRADE_PATTERN=p_800_upgrade_
P_ALL_UPGRADE_PATTERN=p_500_upgrade_
UPGRADE_PATTERN_500=p_500_upgrade
UPGRADE_PATTERN_800=p_800_upgrade
UPGRADE_PATTERN=p_500_upgrade
TARGET_MAIN_EXTRA=extra
DATAFLASHDEV=/dev/mtd3
RAMDISKDEV=/dev/mtd0
TOOLSDIR=/usr/local/bin
PRINTTOOL=${TOOLSDIR}/oled_print
UPGRADE=/mnt/datas
DEV_BLOCK1=/dev/mtdblock1
DEV_BLOCK2=/dev/mtdblock2
DEV_CONFIG=/mnt/config
EXTRA_DIR=extra

# +--------------------------------------------------------------------------+
# | Functions                                                                |
# +--------------------------------------------------------------------------+

# +--------------------------------------------------------------------------+
# |                                                                          |
# +--------------------------------------------------------------------------+

PATH=/usr/local/bin:${PATH}
TARTOOL=$(which tar) > /dev/null 2>&1

#
# Check if upgrade file present in /mnt/datas.
# if yes, MOUNTPOINT is datas else USB
#
mount | grep "${UPGRADE}" > /dev/null 2>&1
if [ ! $? -eq 0 ];
then
	#
	# Mount the device and test if the mount operation succeeds
	#
	echo "mount ${UPGRADE}"
	mount -t jffs2 ${DEV_BLOCK2} ${UPGRADE} > /dev/null 2>&1
	#
	# Check the mount point.
	#
	if [ ! $? -eq 0 ];
	then
		if [ -f ${PRINTTOOL} ];
		then
			${PRINTTOOL} --x 16 --y 30 --text "datas error" --clear
		fi
		exit 50
	fi	
fi

#
# Check if upgrade file present in /mnt/datas.
# if yes, MOUNTPOINT is datas else USB
#
if [ -f ${UPGRADE}/${P_500_UPGRADE_PATTERN}*.tar.bz2 ];
then
	echo "Upgrade file is present in ${UPGRADE}"
	MOUNTPOINT=${UPGRADE}
	P_ALL_UPGRADE_PATTERN=${P_500_UPGRADE_PATTERN}
	UPGRADE_PATTERN=${UPGRADE_PATTERN_500}
else
	if [ -f ${UPGRADE}/${P_800_UPGRADE_PATTERN}*.tar.bz2 ];
	then
		echo "Upgrade file is present in ${UPGRADE}"
		MOUNTPOINT=${UPGRADE}
		P_ALL_UPGRADE_PATTERN=${P_800_UPGRADE_PATTERN}
		UPGRADE_PATTERN=${UPGRADE_PATTERN_800}
	else
		MOUNTPOINT=/mnt/usbdisk
	fi
fi

#
# Start script
#
echo "Starting USB script at " $(date) >> ${MOUNTPOINT}/${LOGFILE} 2>&1

#
# Remove the file containing the list of files found on the device
#
if [ -f ${MOUNTPOINT}/${LISTFILE} ];
then
  rm -f ${MOUNTPOINT}/${LISTFILE} > /dev/null 2>&1
fi

#
# List the upgrade files found on the device
#
ls -l ${MOUNTPOINT}/${P_500_UPGRADE_PATTERN}*.tar.bz2 >> /dev/null 2>&1
if [ ! $? -eq 0 ];
then
	ls -l ${MOUNTPOINT}/${P_800_UPGRADE_PATTERN}*.tar.bz2 >> /dev/null 2>&1
	if [ ! $? -eq 0 ];
	then
		echo "No upgrade file found" >> ${MOUNTPOINT}/${LOGFILE}
		echo "Ending USB script at " $(date) >> ${MOUNTPOINT}/${LOGFILE}
		echo "Exit code is 4" >> ${MOUNTPOINT}/${LOGFILE}
		exit 50
	else
		UPGRADE_PATTERN=${UPGRADE_PATTERN_800}
		P_ALL_UPGRADE_PATTERN=${P_800_UPGRADE_PATTERN}
	fi
else
	UPGRADE_PATTERN=${UPGRADE_PATTERN_500}
	P_ALL_UPGRADE_PATTERN=${P_500_UPGRADE_PATTERN}
fi

#
# List all file which match upgrade pattern.
# eg:	p_500_upgrade-extra-0.0.1.tar.bz2

#
ls ${MOUNTPOINT}/${UPGRADE_PATTERN}-*.tar.bz2 > ${MOUNTPOINT}/${LISTFILE}
#
# Check for the number of .tar.bz2 files found on the USB key.
# If there are no files, then upgrade procedure is aborted.
#
if [ $(cat ${MOUNTPOINT}/${LISTFILE} | wc -l) -eq 0 ];
then
	echo "No upgrade file found" >> ${MOUNTPOINT}/${LOGFILE}
	echo "Ending USB script at " $(date) >> ${MOUNTPOINT}/${LOGFILE}
	echo "Exit code is 5" >> ${MOUNTPOINT}/${LOGFILE}
	exit 50
fi

# +==========================================================================+
# +--------------------------------------------------------------------------+
# | UPGRADING EXTRA DATA for SERVICE LAYER                                   |
# +--------------------------------------------------------------------------+
# +==========================================================================+
grep "${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}" ${MOUNTPOINT}/${LISTFILE} > /dev/null 2>&1
if [ $? -eq 0 ];
then
	#
	# Look for flash memory upgrade file (outside any filesystem).
	#
	TARGET=$(ls -rt ${MOUNTPOINT}/${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}*.tar.bz2 | tail -1 | awk -F- '{print $2}')
	VERSION=$(ls -rt ${MOUNTPOINT}/${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}*.tar.bz2 | tail -1 | awk -F- '{print $3}' | sed -e 's/\.tar.bz2$//g')
	
	if [ -z "${TARGET}" -o -z "${VERSION}" ]
	then
		echo "No valid upgrade file found for flash memory upgrade" >> ${MOUNTPOINT}/${LOGFILE}
	else
		echo "Valid upgrade file found. Processing..." >> ${MOUNTPOINT}/${LOGFILE} 
		echo "Target is '${TARGET}', version is '${VERSION}'" >> ${MOUNTPOINT}/${LOGFILE}
		echo "Processing file ${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}-${VERSION}.tar.bz2" >> ${MOUNTPOINT}/${LOGFILE}
	
		TEMP_DIR=tmp_df_$$
		if [ -d ${MOUNTPOINT}/${TEMP_DIR} ];
		then
			rm -f ${MOUNTPOINT}/${TEMP_DIR}
		fi
		mkdir ${MOUNTPOINT}/${TEMP_DIR}
	
		echo "Uncompressing archive file '${MOUNTPOINT}/${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}-${VERSION}.tar.bz2 '" >> ${MOUNTPOINT}/${LOGFILE} 2>&1
		${TARTOOL} xjf ${MOUNTPOINT}/${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}-${VERSION}.tar.bz2 -C ${MOUNTPOINT}/${TEMP_DIR} >> ${MOUNTPOINT}/${LOGFILE} 2>&1
		
		if [ ! $? -eq 0 ];
		then
			#
			# Remove extracted files
			#
			rm -rf ${MOUNTPOINT}/${TEMP_DIR}
			rm -f ${MOUNTPOINT}/${LISTFILE}
			rm -f ${MOUNTPOINT}/${UPGRADE_PATTERN}-*.tar.bz2
			#
            echo "Error to extract archive !!!" >> ${MOUNTPOINT}/${LOGFILE}
			echo "Ending USB script at " $(date) >> ${MOUNTPOINT}/${LOGFILE}
			echo "Exit code is 5" >> ${MOUNTPOINT}/${LOGFILE}
			exit 5
		fi
	
		# Check for the extra files
		if [ ! -d ${MOUNTPOINT}/${TEMP_DIR}/${EXTRA_DIR} ];
		then
			echo "Extra directory not found !" >> ${MOUNTPOINT}/${LOGFILE}
		else
			if [ -f ${PRINTTOOL} ];
			then
				${PRINTTOOL} --x 40 --y 20 --text "Writing" --clear
				${PRINTTOOL} --x 20 --y 42 --text "Extra Data"
			fi

			mount | grep "${DEV_CONFIG}" > /dev/null 2>&1
			if [ $? -eq 1 ];
			then
				mount -t jffs2 ${DEV_BLOCK1} ${DEV_CONFIG}
			fi

			chmod 777 ${MOUNTPOINT}/${TEMP_DIR}/${EXTRA_DIR}/*
			cp -R -f ${MOUNTPOINT}/${TEMP_DIR}/${EXTRA_DIR}/* ${DEV_CONFIG}
		fi
		
		rm -rf ${MOUNTPOINT}/${TEMP_DIR}/${EXTRA_DIR}
		rm -rf ${MOUNTPOINT}/${TEMP_DIR}
		rm -f ${MOUNTPOINT}/${UPGRADE_PATTERN}-${TARGET_MAIN_EXTRA}-${VERSION}.tar.bz2 
		#${FLASHTOOL} --add=0x3927D --size=1 --data=FF
	fi
fi

#
# Remove temporary resource
#
rm -f ${MOUNTPOINT}/${LISTFILE}

${PRINTTOOL} --x=20 --y=32 --text="Upgrade done" --clear

echo "Ending USB script at " $(date) >> ${MOUNTPOINT}/${LOGFILE}
echo "Exit code is 0" >> ${MOUNTPOINT}/${LOGFILE}

# 
# Terminate the script with success return code.
#
# if MOUNTPOINT == /mnt/datas
if [ ${MOUNTPOINT} == ${UPGRADE} ];
then
	echo "MOUNTPOINT == /mnt/datas"
	#
	# if upgrade file is present,remove it.
	#
	rm -f ${MOUNTPOINT}/*.tar.bz2
	echo "upgrade_script  Upgrade file is removed."
fi
exit 0
# +--------------------------------------------------------------------------+
# | End of script.                                                           |
# +--------------------------------------------------------------------------+
